import netgen.gui 
from netgen.geom2d import unit_square 
from ngsolve import * 
from math import pi 
 
mesh = Mesh(unit_square.GenerateMesh(maxh=0.1)) 
#mesh.Refine() 
 
Draw(mesh) 
mesh.nv, mesh.ne   # number of vertices & elements 
#mesh.GetBoundaries() 
k = 3
V = VectorL2(mesh,order=k, dirichlet ='left|top|bottom|right') 
Q = L2(mesh,order=k-1) 
Vbar = FacetFESpace(mesh, order = k, dirichlet= "bottom|top|left|right") 
Qbar = FacetFESpace(mesh, order = k-1)   
X    = FESpace([V, Vbar, Vbar, Q, Qbar], dgjumps = True) 
 
gfu = GridFunction(X) 
 
(u, ubar1, ubar2, p, pbar)  = X.TrialFunction() 
(v, vbar1, vbar2, q, qbar)  = X.TestFunction() 
n = specialcf.normal(mesh.dim)  
h = specialcf.mesh_size         
ubar = CoefficientFunction((ubar1, ubar2)) 
vbar = CoefficientFunction((vbar1, vbar2)) 
# Penalty parameter 
eta = 10 
#gradient u, v 
du = grad(u) 
dv = grad(v) 
#normal gradient 
dudn=du*n 
dvdn=dv*n 
# Bilinear form 
a = BilinearForm(X) 
ah_dx = InnerProduct(du, dv) 
ah_ds   = (eta/h)*(u-ubar)*(v-vbar) - InnerProduct(u-ubar,dvdn) - InnerProduct(dudn,v-vbar) 
 
bh_dx = -p*div(v) - q*div(u) 
# bh_ds   = - (v-vbar)*n*pbar - (u-ubar)*n*qbar 
bh_ds   = v*n*pbar + u*n*qbar
 
ch = -h * (p-pbar) * (q-qbar) 
ch2 = -h * ( (p-pbar)*(q-qbar) + (p.Other(bnd=0)-pbar)*(q.Other(bnd=0)-qbar) )
 
a += (ah_dx + bh_dx)*dx 
a += (ah_ds + bh_ds)*dx(element_boundary=True) 

# small regularization so we get an invertible matrix
a += -1e-6 * p * q * dx

if True: # with element_boundary
    a += ch*dx(element_boundary=True) # boundary 
else: # with skeleton
    a += ch2 * dx(skeleton=True) # interface 
    a += ch * ds(skeleton=True) # boundary 
 
 
# Linear form 
zeta = x**2*(1-x)**2*y**2*(1-y)**2
u_ex = CoefficientFunction((zeta.Diff(y),-zeta.Diff(x)))
p_ex = x**5+y**5-1/3
f_1 = -(u_ex[0].Diff(x).Diff(x) + u_ex[0].Diff(y).Diff(y)) + p_ex.Diff(x)
f_2 = -(u_ex[1].Diff(x).Diff(x) + u_ex[1].Diff(y).Diff(y)) + p_ex.Diff(y)
f = LinearForm(X) 
f += InnerProduct(CoefficientFunction((f_1,f_2)), v) * dx
 
a.Assemble() 
f.Assemble() 

# print(a.mat)
# quit()

# gfu.components[0].Set( CoefficientFunction((y*(1-y), 0)), BND )
# gfu.components[1].Set( y*(1-y), BND )
 
inv = a.mat.Inverse(freedofs=X.FreeDofs(), inverse="umfpack") 
res = f.vec.CreateVector() 
res.data = f.vec - a.mat*gfu.vec 
gfu.vec.data += inv * res

Draw(gfu.components[3], mesh, "p")
Draw(p_ex, mesh, "p_ex")
Draw(p_ex - gfu.components[3], mesh, "err_p")
Draw(gfu.components[0], mesh, "u")
Draw(u_ex - gfu.components[0], mesh, "err_u")
# Draw(gfu.components[0], mesh, "v")
# Draw(gfu.components[3], mesh, "p")
